package com.sun.xml.internal.ws.spi.db;

import javax.xml.bind.Marshaller;
import javax.xml.bind.JAXBContext;
import com.sun.xml.internal.ws.db.glassfish.JAXBRIContextFactory;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import com.sun.xml.internal.ws.util.ServiceConfigurationError;
import com.sun.xml.internal.ws.util.ServiceFinder;
import java.util.Iterator;
import java.util.logging.Logger;

public abstract class BindingContextFactory
{
    public static final String DefaultDatabindingMode = "glassfish.jaxb";
    public static final String JAXB_CONTEXT_FACTORY_PROPERTY;
    public static final Logger LOGGER;
    
    public BindingContextFactory() {
    }
    
    public static Iterator<BindingContextFactory> serviceIterator() {
        ServiceFinder<BindingContextFactory> sf = ServiceFinder.find(BindingContextFactory.class);
        final Iterator<BindingContextFactory> ibcf = sf.iterator();
        return new Iterator<BindingContextFactory>() {
            private BindingContextFactory bcf;
            
            @Override
            public boolean hasNext() {
                Label_0000: {
                    try {
                        if (ibcf.hasNext()) {
                            this.bcf = (BindingContextFactory)ibcf.next();
                            return true;
                        }
                        return false;
                    }
                    catch (ServiceConfigurationError e) {
                        BindingContextFactory.LOGGER.warning("skipping factory: ServiceConfigurationError: " + e.getMessage());
                        continue;
                    }
                    catch (NoClassDefFoundError ncdfe) {
                        BindingContextFactory.LOGGER.fine("skipping factory: NoClassDefFoundError: " + ncdfe.getMessage());
                        continue;
                    }
                }
            }
            
            @Override
            public BindingContextFactory next() {
                if (BindingContextFactory.LOGGER.isLoggable(Level.FINER)) {
                    BindingContextFactory.LOGGER.finer("SPI found provider: " + this.bcf.getClass().getName());
                }
                return this.bcf;
            }
            
            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }
    
    private static List<BindingContextFactory> factories() {
        List<BindingContextFactory> factories = new ArrayList();
        Iterator<BindingContextFactory> ibcf = serviceIterator();
        while (ibcf.hasNext()) {
            factories.add(ibcf.next());
        }
        if (factories.isEmpty()) {
            if (BindingContextFactory.LOGGER.isLoggable(Level.FINER)) {
                BindingContextFactory.LOGGER.log(Level.FINER, "No SPI providers for BindingContextFactory found, adding: " + JAXBRIContextFactory.class.getName());
            }
            factories.add(new JAXBRIContextFactory());
            return factories;
        }
        else {
            return factories;
        }
    }
    
    protected abstract BindingContext newContext(JAXBContext p0);
    
    protected abstract BindingContext newContext(BindingInfo p0);
    
    protected abstract boolean isFor(String p0);
    
    @Deprecated
    protected abstract BindingContext getContext(Marshaller p0);
    
    private static BindingContextFactory getFactory(String mode) {
        for (BindingContextFactory f : factories()) {
            if (f.isFor(mode)) {
                return f;
            }
            continue;
        }
        return null;
    }
    
    public static BindingContext create(JAXBContext context) throws DatabindingException {
        return getJAXBFactory(context).newContext(context);
    }
    
    public static BindingContext create(BindingInfo bi) {
        String mode = bi.getDatabindingMode();
        if (mode != null) {
            if (BindingContextFactory.LOGGER.isLoggable(Level.FINE)) {
                BindingContextFactory.LOGGER.log(Level.FINE, "Using SEI-configured databindng mode: " + mode);
            }
        }
        else if ((mode = System.getProperty("BindingContextFactory")) != null) {
            bi.setDatabindingMode(mode);
            if (BindingContextFactory.LOGGER.isLoggable(Level.FINE)) {
                BindingContextFactory.LOGGER.log(Level.FINE, "Using databindng: " + mode + " based on 'BindingContextFactory' System property");
            }
        }
        else if ((mode = System.getProperty(BindingContextFactory.JAXB_CONTEXT_FACTORY_PROPERTY)) != null) {
            bi.setDatabindingMode(mode);
            if (BindingContextFactory.LOGGER.isLoggable(Level.FINE)) {
                BindingContextFactory.LOGGER.log(Level.FINE, "Using databindng: " + mode + " based on '" + BindingContextFactory.JAXB_CONTEXT_FACTORY_PROPERTY + "' System property");
            }
        }
        else {
            Iterator<BindingContextFactory> iterator = factories().iterator();
            if (iterator.hasNext()) {
                BindingContextFactory factory = (BindingContextFactory)iterator.next();
                if (BindingContextFactory.LOGGER.isLoggable(Level.FINE)) {
                    BindingContextFactory.LOGGER.log(Level.FINE, "Using SPI-determined databindng mode: " + factory.getClass().getName());
                }
                return factory.newContext(bi);
            }
            else {
                BindingContextFactory.LOGGER.log(Level.SEVERE, "No Binding Context Factories found.");
                throw new DatabindingException("No Binding Context Factories found.");
            }
        }
        BindingContextFactory f = getFactory(mode);
        if (f != null) {
            return f.newContext(bi);
        }
        BindingContextFactory.LOGGER.severe("Unknown Databinding mode: " + mode);
        throw new DatabindingException("Unknown Databinding mode: " + mode);
    }
    
    public static boolean isContextSupported(Object o) {
        if (o == null) {
            return false;
        }
        String pkgName = o.getClass().getPackage().getName();
        for (BindingContextFactory f : factories()) {
            if (f.isFor(pkgName)) {
                return true;
            }
            continue;
        }
        return false;
    }
    
    static BindingContextFactory getJAXBFactory(Object o) {
        String pkgName = o.getClass().getPackage().getName();
        BindingContextFactory f = getFactory(pkgName);
        if (f != null) {
            return f;
        }
        throw new DatabindingException("Unknown JAXBContext implementation: " + o.getClass());
    }
    
    @Deprecated
    public static BindingContext getBindingContext(Marshaller m) {
        return getJAXBFactory(m).getContext(m);
    }
    
    static {
        JAXB_CONTEXT_FACTORY_PROPERTY = BindingContextFactory.class.getName();
        LOGGER = Logger.getLogger(BindingContextFactory.class.getName());
    }
}
